#import "ModelSwarm.h"
#import "Boid.h"
#import "Vector.h"
#import <random.h>
#import <objectbase.h>

#import "globals.h"


@implementation ModelSwarm

+ create: aZone {

  id <ProbeMap> probeMap;
  ModelSwarm *obj;

  obj = [super create: aZone];

  obj->numAgents = 55;
  obj->numObstacles = 3;
  obj->agentID   = 0;
  obj->worldXSize = 500;
  obj->worldYSize = 500;

  obj->boidRaster = nil;

  obj->useBoidCanvas = YES;
  obj->useBoidRaster = YES;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "numAgents"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numObstacles"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldXSize"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldYSize"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "useBoidCanvas"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "useBoidRaster"
                                    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  return obj;

}



///////////////////////////////////////////////////////
//
// buildObjects
//
///////////////////////////////////////////////////////
- buildObjects {

  int i, type;
  double x, y;
  double vx, vy;
  double radius=0;
  Vector *position   = [Vector create: self];
  Vector *velocity   = [Vector create: self];

  [super buildObjects];

  // build the list to keep track of the agents
  objectList  = [List create: self];

  if (boidRaster != nil) 
    {
      [self buildBoidSpace];
    }


  // build the agents
  for (i = 0; i < numAgents; i++) {
    x = [uniformDblRand getDoubleWithMin: 0.0 withMax: (double) worldXSize-1];
    y = [uniformDblRand getDoubleWithMin: 0.0 withMax: (double) worldYSize-1];
    
    vx = [uniformDblRand getDoubleWithMin: -5.0 withMax: 5.0];
    vy = [uniformDblRand getDoubleWithMin: -5.0 withMax: 5.0];

    //if you want Predators, here's where to do the branching.
    //type = [uniformIntRand getIntegerWithMin: 0 withMax: NUMBOIDTYPES-1];
    type = [uniformIntRand getIntegerWithMin: 0 withMax: 0];

    if (type == PREY)
      {
	Boid * agent;
	agent = [Boid createBegin: self];
    
	[agent initModel: self];
	agent=  [agent createEnd];

	radius = 10.0;
	
	[position init: x : y];
	[velocity init: vx : vy];

	
	[agent init: ++agentID Type: type
	       Position: position 
	       Velocity: velocity 
	       Radius: radius];
	if(canvas) [agent createCanvasDisplayItem];
	
	[objectList addLast: agent];
      }

    else //Here we'd use a Predator class if we had one, until then, same old Boid
      {
	Boid * agent;
	agent = [Boid createBegin: self];
	[agent initModel: self];
	agent=  [agent createEnd];
	radius =  20.0;
	
	[position init: x : y];
	[velocity init: vx : vy];
	
	
	[agent init: ++agentID Type: type
	       Position: position 
	       Velocity: velocity 
	       Radius: radius];
	if(canvas) [agent createCanvasDisplayItem];
	
	[objectList addLast: agent];
      }  
  }  // for numAgents


  //
  // Now create numObstacles obstacles, might as well position them at random in the 
  // middle part of the space
  //
  if (1) 
    {
      for (i=0; i<numObstacles; i++)
	{
	  SimObject * agent;
	  
	  x = [uniformDblRand getDoubleWithMin: 0.25*worldXSize withMax: (double) 0.75*worldXSize];
	  y = [uniformDblRand getDoubleWithMin: 0.25*worldYSize withMax: 0.75*worldYSize];
	  
	  [velocity init]; // reset to {0.0, 0.0}
	  
	  agent = [SimObject createBegin: self];
	  [agent init: ++agentID Type: OBSTACLE
		 Position: [position init: x : y] Velocity: velocity Radius: 20.0];
	  [agent initModel: self];
	  if(canvas) [agent createCanvasDisplayItem];
	  agent = [agent createEnd];
	  [objectList addLast: agent];
	}
    }    
  [position   drop];
  [velocity   drop];

  return self;
}


//////////////////////////////////////////////////////
//
// buildBoidSpace
//
//////////////////////////////////////////////////////
- buildBoidSpace {

  //
  // Build the discrete2d space
  //
  boidSpace = [BoidSpace createBegin: self];

  [boidSpace setSizeX: worldXSize Y: worldYSize];
  boidSpace = [boidSpace createEnd];

  [boidSpace setBoidRaster: boidRaster];
  [boidSpace setBoids: objectList];
  return self;

}


//////////////////////////////////////////////////////////
//
// buildActions
//
/////////////////////////////////////////////////////////
- buildActions {

  [super buildActions];

  modelSchedule = [Schedule createBegin: self];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];

  xfprint(objectList);

  [modelSchedule at: 0 createActionForEach: objectList message: M(step)];
  //[modelSchedule at: 0 createActionTo: self message: M(dropDead)];


  return self;

}

- activateIn: swarmContext {
  [super activateIn: swarmContext];
  [modelSchedule activateIn: self];
  return [self getActivity];
}




//////////////////////////////////////////////////////////
////      SET
/////////////////////////////////////////////////////////
- setBoidRaster: (id <Raster>) aRaster 
{
  boidRaster = aRaster;
  return self;
}
    



///////////////////////////////////////////
// setBoidCanvas
///////////////////////////////////////////
- setBoidCanvas: (id <Canvas>) c
{
  canvas = c;
  return self;
}



/////////////////////////////////////////////
// getCanvas
/////////////////////////////////////////////
- (id <Canvas>) getBoidCanvas
{
  return canvas;
}


/////////////////////////////////////////
//
// display
//
////////////////////////////////////////

- displayCanvasObjects
 {
   //pj pj fix!
    if (canvas)
      {
        //[objectList forEach: @selector(display)];
        id object;
        id <Index> index = [objectList begin: self];
        while ( (object = [index next] ) )
 	 [object displayOnCanvas];
      }
   return self;
 }




/////////////////////////////////////////////////
//
// getObjectList
//
////////////////////////////////////////////////
- (id <List>) getObjectList { 
 
        return objectList;

}



/////////////////////////////////////////////////////
//
// getWorldSize 
//
///////////////////////////////////////////////////
- (int) getWorldXSize { return worldXSize;}
- (int) getWorldYSize { return worldYSize;}



///////////////////////////////////////////////////
//
// getBoidSpace
//
//////////////////////////////////////////////////
- getBoidSpace {

   return boidSpace;

}


//////////////////////////////////////////////////
//
// getBoids
//
//////////////////////////////////////////////////
- getBoids {

   return objectList;

}



////////////////////////////////////////////////
//
// useBoidRaster
//
////////////////////////////////////////////////
- (BOOL) useBoidRaster 
{
   return useBoidRaster;
}

////////////////////////////////////////////////
//
// useBoidCanvas
//
////////////////////////////////////////////////
- (BOOL) useBoidCanvas {

   return useBoidCanvas;

}
@end

