#import <simtoolsgui.h>
#import "ObserverSwarm.h"


@implementation ObserverSwarm: GUISwarm

+ createBegin: aZone
{
  ObserverSwarm * obj;
  obj = [super createBegin: aZone];
  return obj;
}

- createEnd
{
  return [super createEnd];
}



//////////////////////////////////////////////////
//
// buildObjects
//
//////////////////////////////////////////////////
- buildObjects
{
 
  [super buildObjects];
  
  modelSwarm = [ModelSwarm create: self];
  
  CREATE_PROBE_DISPLAY (modelSwarm);
  //  CREATE_PROBE_DISPLAY (self);
  [controlPanel setStateStopped];

  //boidRaster is Yes, so create one
  if ( [modelSwarm useBoidRaster] == YES )
    {      

      id <Colormap> colorMap;   


      colorMap = [Colormap create: self];
      
      [colorMap setColor: 0 ToName: "white"];
      [colorMap setColor: 1 ToName: "green"];
      [colorMap setColor: 2 ToName: "blue"];
      [colorMap setColor: 3 ToName: "black"];
      [colorMap setColor: 4 ToName: "red"];
      [colorMap setColor: 5 ToName: "purple"];


      boidRaster = [ZoomRaster createBegin: self];
     
      boidRaster = [boidRaster createEnd];
      SET_WINDOW_GEOMETRY_RECORD_NAME (boidRaster);
      //      [boidRaster enableDestroyNotification: self
      //		  notificationMethod: @selector (_boidRasterDeath_:)];

      [boidRaster setColormap: colorMap];
      
      [boidRaster setZoomFactor: 1];
      
      [boidRaster setWidth: [modelSwarm getWorldXSize] 
      		  Height: [modelSwarm getWorldYSize]];
      
     
      [boidRaster setWindowTitle: "Boid World"];
      
      [boidRaster pack];				
      
      [modelSwarm setBoidRaster: boidRaster];
    }

 
 
  
  // if you want a canvas, you have to make one
  if ( [modelSwarm useBoidCanvas] == YES)
    {
      canvas = [Canvas create: globalZone];
      [canvas setWidth: [modelSwarm getWorldXSize] 
	      Height: [modelSwarm getWorldYSize]];
      [canvas setWindowTitle: "Flock World"];
      [canvas pack];
      
      [modelSwarm setBoidCanvas: canvas];
    }


  // modelSwarm can build its objects, because now it already
  // has been given the canvas or raster.
  [modelSwarm buildObjects];

  //now modelSwarm has objects, we can setup an Object2Display that
  //relies on the collections
  if (boidRaster)
    {
      boidDisplay = [Object2dDisplay create: self
    				     setDisplayWidget: boidRaster
    				     setDiscrete2dToDisplay: [modelSwarm getBoidSpace]
    				     setDisplayMessage: M(drawSelfOn:)]; 
      [boidDisplay setObjectCollection: [modelSwarm getBoids]];
    }




  if (0)
    { 
      popGraph = [EZGraph createBegin: [self getZone]];

      //  SET_WINDOW_GEOMETRY_RECORD_NAME (popGraph);
      [popGraph setTitle: "Poplulation vs. time"];
      [popGraph setAxisLabelsX: "time" Y: "population"];
      popGraph = [popGraph createEnd];
      
      [popGraph createSequence: "Population"
		withFeedFrom: [modelSwarm getObjectList] 
		andSelector: M(getCount)];
    }
  return self;
}  




///////////////////////////////////////////////////////////
// 
// buildActions
//
//////////////////////////////////////////////////////////
- buildActions
{
  //  printf ("ObserverSwarm\tbuildActions\n");
  [super buildActions];
  
  
  [modelSwarm buildActions];

  displayActions = [ActionGroup create: [self getZone]];
  //
  //[displayActions createActionTo: popGraph    message: M(step)];

  [displayActions createActionTo: actionCache message: M(doTkEvents)]; 
  [displayActions createActionTo: modelSwarm  message: M(displayCanvasObjects)];
  [displayActions createActionTo: self message: M(updateRaster)];

  // then actually schedule the display actions
  displaySchedule = [Schedule createBegin: [self getZone]];
  [displaySchedule setRepeatInterval: 1];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];

  return self;
}  

//////////////////////////////////////////////////////
//
// updateRaster
//
/////////////////////////////////////////////////////
- updateRaster {

   if (boidRaster) {
       [boidRaster erase];
       [boidDisplay display];
       [boidRaster drawSelf];
    }

   return self;

}


//////////////////////////////////////////////////////////
//
// activateIn
//
////////////////////////////////////////////////////////
- activateIn: swarmContext
{
  [super activateIn: swarmContext];
  [modelSwarm activateIn: self];
  [displaySchedule activateIn: self];
  
  return [self getActivity];
}



/////////////////////////////////////////////////////
//
// getDisplay
//
//////////////////////////////////////////////////////
- getDisplay
{
  return canvas;
}
@end
